@@ -25,6 +25,12 @@
 
 #include <FLAC/all.h>
 
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
+#define LEGACY_FLAC
+#else
+#undef LEGACY_FLAC
+#endif 
+
 G_BEGIN_DECLS
 
 #define GST_TYPE_FLAC_ENC (gst_flac_enc_get_type())
@@ -39,7 +45,12 @@ typedef struct _GstFlacEncClass GstFlacEncClass;
 struct _GstFlacEnc {
   GstElement     element;
 
-  GstPad *sinkpad,*srcpad;
+  GstPad        *sinkpad;
+  GstPad        *srcpad;
+
+  GstFlowReturn  last_flow; /* save flow from last push so we can pass the
+                             * correct flow return upstream in case the push
+                             * fails for some reason */
 
   gboolean       first;
   GstBuffer     *first_buf;
@@ -54,10 +65,18 @@ struct _GstFlacEnc {
   gboolean       stopped;
   FLAC__int32   *data;
 
+#if !defined(FLAC_API_VERSION_CURRENT) || FLAC_API_VERSION_CURRENT < 8
   FLAC__SeekableStreamEncoder *encoder;
+#else
+  FLAC__StreamEncoder *encoder;
+#endif
   FLAC__StreamMetadata **meta;
 
   GstTagList *     tags;
+
+  /* queue headers until we have them all so we can add streamheaders to caps */
+  gboolean         got_headers;
+  GList           *headers;
 };
 
 struct _GstFlacEncClass {
