@@ -21,20 +21,28 @@
 
 /**
  * SECTION:element-pulsesink
+ * @short_description: Output audio to a PulseAudio sound server
  * @see_also: pulsesrc, pulsemixer
  *
- * This element outputs audio to a
- * <ulink href="http://www.pulseaudio.org">PulseAudio sound server</ulink>.
- *
  * <refsect2>
+ * <para>
+ * This element outputs audio to a PulseAudio sound server.
+ * </para>
  * <title>Example pipelines</title>
- * |[
+ * <para>
+ * <programlisting>
  * gst-launch -v filesrc location=sine.ogg ! oggdemux ! vorbisdec ! audioconvert ! audioresample ! pulsesink
- * ]| Play an Ogg/Vorbis file.
- * |[
+ * </programlisting>
+ * Play an Ogg/Vorbis file.
+ * </para>
+ * <para>
+ * <programlisting>
  * gst-launch -v audiotestsrc ! audioconvert ! volume volume=0.4 ! pulsesink
- * ]| Play a 440Hz sine wave.
+ * </programlisting>
+ * Play a 440Hz sine wave.
+ * </para>
  * </refsect2>
+ *
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,19 +61,14 @@
 GST_DEBUG_CATEGORY_EXTERN (pulse_debug);
 #define GST_CAT_DEFAULT pulse_debug
 
-/* according to
- * http://www.pulseaudio.org/ticket/314
- * we need pulse-0.9.12 to use sink volume properties
- */
-
 enum
 {
   PROP_SERVER = 1,
   PROP_DEVICE,
-  PROP_DEVICE_NAME,
-  PROP_VOLUME
 };
 
+static GstAudioSinkClass *parent_class = NULL;
+
 static void gst_pulsesink_destroy_stream (GstPulseSink * pulsesink);
 
 static void gst_pulsesink_destroy_context (GstPulseSink * pulsesink);
@@ -76,6 +79,8 @@ static void gst_pulsesink_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_pulsesink_finalize (GObject * object);
 
+static void gst_pulsesink_dispose (GObject * object);
+
 static gboolean gst_pulsesink_open (GstAudioSink * asink);
 
 static gboolean gst_pulsesink_close (GstAudioSink * asink);
@@ -92,61 +97,12 @@ static void gst_pulsesink_reset (GstAudioSink * asink);
 
 static gboolean gst_pulsesink_event (GstBaseSink * sink, GstEvent * event);
 
-static GstStateChangeReturn gst_pulsesink_change_state (GstElement *
-    element, GstStateChange transition);
-
-static void gst_pulsesink_init_interfaces (GType type);
-
-static gboolean gst_pulsesink_is_dead (GstPulseSink * pulsesink);
-
 #if (G_BYTE_ORDER == G_LITTLE_ENDIAN)
 # define ENDIANNESS   "LITTLE_ENDIAN, BIG_ENDIAN"
 #else
 # define ENDIANNESS   "BIG_ENDIAN, LITTLE_ENDIAN"
 #endif
 
-GST_IMPLEMENT_PULSEPROBE_METHODS (GstPulseSink, gst_pulsesink);
-GST_BOILERPLATE_FULL (GstPulseSink, gst_pulsesink, GstAudioSink,
-    GST_TYPE_AUDIO_SINK, gst_pulsesink_init_interfaces);
-
-static gboolean
-gst_pulsesink_interface_supported (GstImplementsInterface *
-    iface, GType interface_type)
-{
-  GstPulseSink *this = GST_PULSESINK (iface);
-
-  if (interface_type == GST_TYPE_PROPERTY_PROBE && this->probe)
-    return TRUE;
-
-  return FALSE;
-}
-
-static void
-gst_pulsesink_implements_interface_init (GstImplementsInterfaceClass * klass)
-{
-  klass->supported = gst_pulsesink_interface_supported;
-}
-
-static void
-gst_pulsesink_init_interfaces (GType type)
-{
-  static const GInterfaceInfo implements_iface_info = {
-    (GInterfaceInitFunc) gst_pulsesink_implements_interface_init,
-    NULL,
-    NULL,
-  };
-  static const GInterfaceInfo probe_iface_info = {
-    (GInterfaceInitFunc) gst_pulsesink_property_probe_interface_init,
-    NULL,
-    NULL,
-  };
-
-  g_type_add_interface_static (type, GST_TYPE_IMPLEMENTS_INTERFACE,
-      &implements_iface_info);
-  g_type_add_interface_static (type, GST_TYPE_PROPERTY_PROBE,
-      &probe_iface_info);
-}
-
 static void
 gst_pulsesink_base_init (gpointer g_class)
 {
@@ -160,30 +116,30 @@ gst_pulsesink_base_init (gpointer g_class)
           "width = (int) 16, "
           "depth = (int) 16, "
           "rate = (int) [ 1, MAX ], "
-          "channels = (int) [ 1, 32 ];"
+          "channels = (int) [ 1, 16 ];"
           "audio/x-raw-float, "
           "endianness = (int) { " ENDIANNESS " }, "
           "width = (int) 32, "
           "rate = (int) [ 1, MAX ], "
-          "channels = (int) [ 1, 32 ];"
+          "channels = (int) [ 1, 16 ];"
           "audio/x-raw-int, "
           "endianness = (int) { " ENDIANNESS " }, "
           "signed = (boolean) TRUE, "
           "width = (int) 32, "
           "depth = (int) 32, "
           "rate = (int) [ 1, MAX ], "
-          "channels = (int) [ 1, 32 ];"
+          "channels = (int) [ 1, 16 ];"
           "audio/x-raw-int, "
           "signed = (boolean) FALSE, "
           "width = (int) 8, "
           "depth = (int) 8, "
           "rate = (int) [ 1, MAX ], "
-          "channels = (int) [ 1, 32 ];"
+          "channels = (int) [ 1, 16 ];"
           "audio/x-alaw, "
           "rate = (int) [ 1, MAX], "
-          "channels = (int) [ 1, 32 ];"
+          "channels = (int) [ 1, 16 ];"
           "audio/x-mulaw, "
-          "rate = (int) [ 1, MAX], " "channels = (int) [ 1, 32 ]")
+          "rate = (int) [ 1, MAX], " "channels = (int) [ 1, 16 ]")
       );
 
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
@@ -196,20 +152,22 @@ gst_pulsesink_base_init (gpointer g_class)
 }
 
 static void
-gst_pulsesink_class_init (GstPulseSinkClass * klass)
+gst_pulsesink_class_init (gpointer g_class, gpointer class_data)
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
-  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
-  GstBaseSinkClass *gstbasesink_class = GST_BASE_SINK_CLASS (klass);
-  GstAudioSinkClass *gstaudiosink_class = GST_AUDIO_SINK_CLASS (klass);
 
+  GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
+
+  GstBaseSinkClass *gstbasesink_class = GST_BASE_SINK_CLASS (g_class);
+
+  GstAudioSinkClass *gstaudiosink_class = GST_AUDIO_SINK_CLASS (g_class);
+
+  parent_class = g_type_class_peek_parent (g_class);
+
+  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_pulsesink_dispose);
   gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_pulsesink_finalize);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_pulsesink_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_pulsesink_get_property);
 
-  gstelement_class->change_state =
-      GST_DEBUG_FUNCPTR (gst_pulsesink_change_state);
-
   gstbasesink_class->event = GST_DEBUG_FUNCPTR (gst_pulsesink_event);
 
   gstaudiosink_class->open = GST_DEBUG_FUNCPTR (gst_pulsesink_open);
@@ -230,54 +188,26 @@ gst_pulsesink_class_init (GstPulseSinkClass * klass)
       g_param_spec_string ("device", "Sink",
           "The PulseAudio sink device to connect to", NULL,
           G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-  g_object_class_install_property (gobject_class,
-      PROP_DEVICE_NAME,
-      g_param_spec_string ("device-name", "Device name",
-          "Human-readable name of the sound device", NULL,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
-#if HAVE_PULSE_0_9_12
-  g_object_class_install_property (gobject_class,
-      PROP_VOLUME,
-      g_param_spec_double ("volume", "Volume",
-          "Volume of this stream", 0.0, 1000.0, 1.0,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
-#endif
 }
 
 static void
-gst_pulsesink_init (GstPulseSink * pulsesink, GstPulseSinkClass * klass)
+gst_pulsesink_init (GTypeInstance * instance, gpointer g_class)
 {
+
+  GstPulseSink *pulsesink = GST_PULSESINK (instance);
+
   int e;
 
-  pulsesink->server = pulsesink->device = pulsesink->stream_name =
-      pulsesink->device_description = NULL;
+  pulsesink->server = pulsesink->device = pulsesink->stream_name = NULL;
 
   pulsesink->context = NULL;
   pulsesink->stream = NULL;
 
-  pulsesink->volume = 1.0;
-  pulsesink->volume_set = FALSE;
-
-#if HAVE_PULSE_0_9_13
-  pa_sample_spec_init (&pulsesink->sample_spec);
-#else
-  pulsesink->sample_spec.format = PA_SAMPLE_INVALID;
-  pulsesink->sample_spec.rate = 0;
-  pulsesink->sample_spec.channels = 0;
-#endif
-
-  pulsesink->operation_success = FALSE;
-  pulsesink->did_reset = FALSE;
-  pulsesink->in_write = FALSE;
-  pulsesink->notify = 0;
-
   pulsesink->mainloop = pa_threaded_mainloop_new ();
   g_assert (pulsesink->mainloop);
 
   e = pa_threaded_mainloop_start (pulsesink->mainloop);
   g_assert (e == 0);
-
-  pulsesink->probe = gst_pulseprobe_new (G_OBJECT (pulsesink), G_OBJECT_GET_CLASS (pulsesink), PROP_DEVICE, pulsesink->device, TRUE, FALSE);    /* TRUE for sinks, FALSE for sources */
 }
 
 static void
@@ -285,21 +215,12 @@ gst_pulsesink_destroy_stream (GstPulseSink * pulsesink)
 {
   if (pulsesink->stream) {
     pa_stream_disconnect (pulsesink->stream);
-
-    /* Make sure we don't get any further callbacks */
-    pa_stream_set_state_callback (pulsesink->stream, NULL, NULL);
-    pa_stream_set_write_callback (pulsesink->stream, NULL, NULL);
-    pa_stream_set_latency_update_callback (pulsesink->stream, NULL, NULL);
-
     pa_stream_unref (pulsesink->stream);
     pulsesink->stream = NULL;
   }
 
   g_free (pulsesink->stream_name);
   pulsesink->stream_name = NULL;
-
-  g_free (pulsesink->device_description);
-  pulsesink->device_description = NULL;
 }
 
 static void
@@ -310,11 +231,6 @@ gst_pulsesink_destroy_context (GstPulseSink * pulsesink)
 
   if (pulsesink->context) {
     pa_context_disconnect (pulsesink->context);
-
-    /* Make sure we don't get any further callbacks */
-    pa_context_set_state_callback (pulsesink->context, NULL, NULL);
-    pa_context_set_subscribe_callback (pulsesink->context, NULL, NULL);
-
     pa_context_unref (pulsesink->context);
     pulsesink->context = NULL;
   }
@@ -331,187 +247,17 @@ gst_pulsesink_finalize (GObject * object)
 
   g_free (pulsesink->server);
   g_free (pulsesink->device);
+  g_free (pulsesink->stream_name);
 
   pa_threaded_mainloop_free (pulsesink->mainloop);
 
-  if (pulsesink->probe) {
-    gst_pulseprobe_free (pulsesink->probe);
-    pulsesink->probe = NULL;
-  }
-
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-#if HAVE_PULSE_0_9_12
 static void
-gst_pulsesink_set_volume (GstPulseSink * pulsesink, gdouble volume)
+gst_pulsesink_dispose (GObject * object)
 {
-  pa_cvolume v;
-  pa_operation *o = NULL;
-
-  pa_threaded_mainloop_lock (pulsesink->mainloop);
-
-  pulsesink->volume = volume;
-  pulsesink->volume_set = TRUE;
-
-  if (!pulsesink->stream)
-    goto unlock;
-
-  gst_pulse_cvolume_from_linear (&v, pulsesink->sample_spec.channels, volume);
-
-  if (!(o = pa_context_set_sink_input_volume (pulsesink->context,
-              pa_stream_get_index (pulsesink->stream), &v, NULL, NULL))) {
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("pa_stream_set_sink_input_volume() failed: %s",
-            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-    goto unlock;
-  }
-
-  /* We don't really care about the result of this call */
-
-unlock:
-
-  if (o)
-    pa_operation_unref (o);
-
-  pa_threaded_mainloop_unlock (pulsesink->mainloop);
-}
-
-static void
-gst_pulsesink_sink_input_info_cb (pa_context * c, const pa_sink_input_info * i,
-    int eol, void *userdata)
-{
-  GstPulseSink *pulsesink = GST_PULSESINK (userdata);
-
-  if (!i)
-    return;
-
-  if (!pulsesink->stream)
-    return;
-
-  g_assert (i->index == pa_stream_get_index (pulsesink->stream));
-
-  pulsesink->volume = pa_sw_volume_to_linear (pa_cvolume_max (&i->volume));
-}
-
-static gdouble
-gst_pulsesink_get_volume (GstPulseSink * pulsesink)
-{
-  pa_operation *o = NULL;
-  gdouble v;
-
-  pa_threaded_mainloop_lock (pulsesink->mainloop);
-
-  if (!pulsesink->stream)
-    goto unlock;
-
-  if (!(o = pa_context_get_sink_input_info (pulsesink->context,
-              pa_stream_get_index (pulsesink->stream),
-              gst_pulsesink_sink_input_info_cb, pulsesink))) {
-
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("pa_stream_get_sink_input_info() failed: %s",
-            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-    goto unlock;
-  }
-
-  while (pa_operation_get_state (o) == PA_OPERATION_RUNNING) {
-
-    if (gst_pulsesink_is_dead (pulsesink))
-      goto unlock;
-
-    pa_threaded_mainloop_wait (pulsesink->mainloop);
-  }
-
-unlock:
-
-  if (o)
-    pa_operation_unref (o);
-
-  v = pulsesink->volume;
-
-  pa_threaded_mainloop_unlock (pulsesink->mainloop);
-
-  return v;
-}
-#endif
-
-static gboolean
-gst_pulsesink_is_dead (GstPulseSink * pulsesink)
-{
-
-  if (!pulsesink->context
-      || !PA_CONTEXT_IS_GOOD (pa_context_get_state (pulsesink->context))
-      || !pulsesink->stream
-      || !PA_STREAM_IS_GOOD (pa_stream_get_state (pulsesink->stream))) {
-    const gchar *err_str = pulsesink->context ?
-        pa_strerror (pa_context_errno (pulsesink->context)) : NULL;
-
-    GST_ELEMENT_ERROR ((pulsesink), RESOURCE, FAILED, ("Disconnected: %s",
-            err_str), (NULL));
-    return TRUE;
-  }
-
-  return FALSE;
-}
-
-static void
-gst_pulsesink_sink_info_cb (pa_context * c, const pa_sink_info * i, int eol,
-    void *userdata)
-{
-  GstPulseSink *pulsesink = GST_PULSESINK (userdata);
-
-  if (!i)
-    return;
-
-  if (!pulsesink->stream)
-    return;
-
-  g_assert (i->index == pa_stream_get_device_index (pulsesink->stream));
-
-  g_free (pulsesink->device_description);
-  pulsesink->device_description = g_strdup (i->description);
-}
-
-static gchar *
-gst_pulsesink_device_description (GstPulseSink * pulsesink)
-{
-  pa_operation *o = NULL;
-  gchar *t;
-
-  pa_threaded_mainloop_lock (pulsesink->mainloop);
-
-  if (!pulsesink->stream)
-    goto unlock;
-
-  if (!(o = pa_context_get_sink_info_by_index (pulsesink->context,
-              pa_stream_get_device_index (pulsesink->stream),
-              gst_pulsesink_sink_info_cb, pulsesink))) {
-
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("pa_stream_get_sink_info() failed: %s",
-            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-    goto unlock;
-  }
-
-  while (pa_operation_get_state (o) == PA_OPERATION_RUNNING) {
-
-    if (gst_pulsesink_is_dead (pulsesink))
-      goto unlock;
-
-    pa_threaded_mainloop_wait (pulsesink->mainloop);
-  }
-
-unlock:
-
-  if (o)
-    pa_operation_unref (o);
-
-  t = g_strdup (pulsesink->device_description);
-
-  pa_threaded_mainloop_unlock (pulsesink->mainloop);
-
-  return t;
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 static void
@@ -524,10 +270,6 @@ gst_pulsesink_set_property (GObject * object,
     case PROP_SERVER:
       g_free (pulsesink->server);
       pulsesink->server = g_value_dup_string (value);
-
-      if (pulsesink->probe)
-        gst_pulseprobe_set_server (pulsesink->probe, pulsesink->server);
-
       break;
 
     case PROP_DEVICE:
@@ -535,12 +277,6 @@ gst_pulsesink_set_property (GObject * object,
       pulsesink->device = g_value_dup_string (value);
       break;
 
-#if HAVE_PULSE_0_9_12
-    case PROP_VOLUME:
-      gst_pulsesink_set_volume (pulsesink, g_value_get_double (value));
-      break;
-#endif
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -563,19 +299,6 @@ gst_pulsesink_get_property (GObject * object,
       g_value_set_string (value, pulsesink->device);
       break;
 
-    case PROP_DEVICE_NAME:{
-      char *t = gst_pulsesink_device_description (pulsesink);
-      g_value_set_string (value, t);
-      g_free (t);
-      break;
-    }
-
-#if HAVE_PULSE_0_9_12
-    case PROP_VOLUME:
-      g_value_set_double (value, gst_pulsesink_get_volume (pulsesink));
-      break;
-#endif
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -602,33 +325,6 @@ gst_pulsesink_context_state_cb (pa_context * c, void *userdata)
   }
 }
 
-#if HAVE_PULSE_0_9_12
-static void
-gst_pulsesink_context_subscribe_cb (pa_context * c,
-    pa_subscription_event_type_t t, uint32_t idx, void *userdata)
-{
-  GstPulseSink *pulsesink = GST_PULSESINK (userdata);
-
-  if (t != (PA_SUBSCRIPTION_EVENT_SINK_INPUT | PA_SUBSCRIPTION_EVENT_CHANGE) &&
-      t != (PA_SUBSCRIPTION_EVENT_SINK_INPUT | PA_SUBSCRIPTION_EVENT_NEW))
-    return;
-
-  if (!pulsesink->stream)
-    return;
-
-  if (idx != pa_stream_get_index (pulsesink->stream))
-    return;
-
-  /* Actually this event is also triggered when other properties of
-   * the stream change that are unrelated to the volume. However it is
-   * probably cheaper to signal the change here and check for the
-   * volume when the GObject property is read instead of querying it always. */
-
-  /* inform streaming thread to notify */
-  g_atomic_int_compare_and_exchange (&pulsesink->notify, 0, 1);
-}
-#endif
-
 static void
 gst_pulsesink_stream_state_cb (pa_stream * s, void *userdata)
 {
@@ -668,13 +364,11 @@ static gboolean
 gst_pulsesink_open (GstAudioSink * asink)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (asink);
+
   gchar *name = gst_pulse_client_name ();
 
   pa_threaded_mainloop_lock (pulsesink->mainloop);
 
-  g_assert (!pulsesink->context);
-  g_assert (!pulsesink->stream);
-
   if (!(pulsesink->context =
           pa_context_new (pa_threaded_mainloop_get_api (pulsesink->mainloop),
               name))) {
@@ -685,10 +379,6 @@ gst_pulsesink_open (GstAudioSink * asink)
 
   pa_context_set_state_callback (pulsesink->context,
       gst_pulsesink_context_state_cb, pulsesink);
-#if HAVE_PULSE_0_9_12
-  pa_context_set_subscribe_callback (pulsesink->context,
-      gst_pulsesink_context_subscribe_cb, pulsesink);
-#endif
 
   if (pa_context_connect (pulsesink->context, pulsesink->server, 0, NULL) < 0) {
     GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED, ("Failed to connect: %s",
@@ -696,24 +386,13 @@ gst_pulsesink_open (GstAudioSink * asink)
     goto unlock_and_fail;
   }
 
-  for (;;) {
-    pa_context_state_t state;
+  /* Wait until the context is ready */
+  pa_threaded_mainloop_wait (pulsesink->mainloop);
 
-    state = pa_context_get_state (pulsesink->context);
-
-    if (!PA_CONTEXT_IS_GOOD (state)) {
-      GST_DEBUG_OBJECT (pulsesink, "Context state was not READY. Got: %d",
-          state);
-      GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED, ("Failed to connect: %s",
-              pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-      goto unlock_and_fail;
-    }
-
-    if (state == PA_CONTEXT_READY)
-      break;
-
-    /* Wait until the context is ready */
-    pa_threaded_mainloop_wait (pulsesink->mainloop);
+  if (pa_context_get_state (pulsesink->context) != PA_CONTEXT_READY) {
+    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED, ("Failed to connect: %s",
+            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
+    goto unlock_and_fail;
   }
 
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
@@ -722,8 +401,6 @@ gst_pulsesink_open (GstAudioSink * asink)
 
 unlock_and_fail:
 
-  gst_pulsesink_destroy_context (pulsesink);
-
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
   g_free (name);
   return FALSE;
@@ -745,49 +422,36 @@ static gboolean
 gst_pulsesink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
 {
   pa_buffer_attr buf_attr;
+
   pa_channel_map channel_map;
+
   GstPulseSink *pulsesink = GST_PULSESINK (asink);
-  pa_operation *o = NULL;
-  pa_cvolume v;
 
   if (!gst_pulse_fill_sample_spec (spec, &pulsesink->sample_spec)) {
     GST_ELEMENT_ERROR (pulsesink, RESOURCE, SETTINGS,
         ("Invalid sample specification."), (NULL));
-    return FALSE;
+    goto unlock_and_fail;
   }
 
   pa_threaded_mainloop_lock (pulsesink->mainloop);
 
-  if (!pulsesink->context) {
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED, ("Bad context"), (NULL));
+  if (!pulsesink->context
+      || pa_context_get_state (pulsesink->context) != PA_CONTEXT_READY) {
+    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED, ("Bad context state: %s",
+            pulsesink->
+            context ? pa_strerror (pa_context_errno (pulsesink->context)) :
+            NULL), (NULL));
     goto unlock_and_fail;
   }
 
-  g_assert (!pulsesink->stream);
-
-  if (!(o =
-          pa_context_subscribe (pulsesink->context,
-              PA_SUBSCRIPTION_MASK_SINK_INPUT, NULL, NULL))) {
-    const gchar *err_str = pulsesink->context ?
-        pa_strerror (pa_context_errno (pulsesink->context)) : NULL;
-
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("pa_context_subscribe() failed: %s", err_str), (NULL));
-    goto unlock_and_fail;
-  }
-
-  pa_operation_unref (o);
-
   if (!(pulsesink->stream = pa_stream_new (pulsesink->context,
-              pulsesink->stream_name ?
-              pulsesink->stream_name : "Playback Stream",
+              pulsesink->
+              stream_name ? pulsesink->stream_name : "Playback Stream",
               &pulsesink->sample_spec,
               gst_pulse_gst_to_channel_map (&channel_map, spec)))) {
-    const gchar *err_str = pulsesink->context ?
-        pa_strerror (pa_context_errno (pulsesink->context)) : NULL;
-
     GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("Failed to create stream: %s", err_str), (NULL));
+        ("Failed to create stream: %s",
+            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
     goto unlock_and_fail;
   }
 
@@ -801,58 +465,39 @@ gst_pulsesink_prepare (GstAudioSink * asink, GstRingBufferSpec * spec)
   memset (&buf_attr, 0, sizeof (buf_attr));
   buf_attr.tlength = spec->segtotal * spec->segsize;
   buf_attr.maxlength = buf_attr.tlength * 2;
-  buf_attr.prebuf = buf_attr.tlength;
+  buf_attr.prebuf = buf_attr.tlength - spec->segsize;
   buf_attr.minreq = spec->segsize;
 
-  if (pulsesink->volume_set)
-    gst_pulse_cvolume_from_linear (&v, pulsesink->sample_spec.channels,
-        pulsesink->volume);
-
   if (pa_stream_connect_playback (pulsesink->stream, pulsesink->device,
           &buf_attr,
-          PA_STREAM_INTERPOLATE_TIMING |
-          PA_STREAM_AUTO_TIMING_UPDATE | PA_STREAM_NOT_MONOTONOUS |
-#if HAVE_PULSE_0_9_11
-          PA_STREAM_ADJUST_LATENCY |
-#endif
-          PA_STREAM_START_CORKED, pulsesink->volume_set ? &v : NULL, NULL) < 0) {
-
+          PA_STREAM_INTERPOLATE_TIMING | PA_STREAM_AUTO_TIMING_UPDATE |
+          PA_STREAM_NOT_MONOTONOUS, NULL, NULL) < 0) {
     GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
         ("Failed to connect stream: %s",
             pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
     goto unlock_and_fail;
   }
 
-  for (;;) {
-    pa_stream_state_t state;
+  /* Wait until the stream is ready */
+  pa_threaded_mainloop_wait (pulsesink->mainloop);
 
-    state = pa_stream_get_state (pulsesink->stream);
-
-    if (!PA_STREAM_IS_GOOD (state)) {
-      GST_DEBUG_OBJECT (pulsesink, "Stream state was not READY. Got: %d",
-          state);
-      GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-          ("Failed to connect stream: %s",
-              pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-      goto unlock_and_fail;
-    }
-
-    if (state == PA_STREAM_READY)
-      break;
-
-    /* Wait until the stream is ready */
-    pa_threaded_mainloop_wait (pulsesink->mainloop);
+  if (pa_stream_get_state (pulsesink->stream) != PA_STREAM_READY) {
+    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
+        ("Failed to connect stream: %s",
+            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
+    goto unlock_and_fail;
   }
 
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
+
+  spec->bytes_per_sample = pa_frame_size (&pulsesink->sample_spec);
+  memset (spec->silence_sample, 0, spec->bytes_per_sample);
+
   return TRUE;
 
 unlock_and_fail:
 
-  gst_pulsesink_destroy_stream (pulsesink);
-
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
-
   return FALSE;
 }
 
@@ -868,26 +513,27 @@ gst_pulsesink_unprepare (GstAudioSink * asink)
   return TRUE;
 }
 
+#define CHECK_DEAD_GOTO(pulsesink, label) \
+if (!(pulsesink)->context || pa_context_get_state((pulsesink)->context) != PA_CONTEXT_READY || \
+    !(pulsesink)->stream || pa_stream_get_state((pulsesink)->stream) != PA_STREAM_READY) { \
+    GST_ELEMENT_ERROR((pulsesink), RESOURCE, FAILED, ("Disconnected: %s", (pulsesink)->context ? pa_strerror(pa_context_errno((pulsesink)->context)) : NULL), (NULL)); \
+    goto label; \
+}
+
 static guint
 gst_pulsesink_write (GstAudioSink * asink, gpointer data, guint length)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (asink);
+
   size_t sum = 0;
 
-  /* FIXME post message rather than using a signal (as mixer interface) */
-  if (g_atomic_int_compare_and_exchange (&pulsesink->notify, 1, 0))
-    g_object_notify (G_OBJECT (pulsesink), "volume");
-
   pa_threaded_mainloop_lock (pulsesink->mainloop);
 
-  pulsesink->in_write = TRUE;
-
   while (length > 0) {
     size_t l;
 
     for (;;) {
-      if (gst_pulsesink_is_dead (pulsesink))
-        goto unlock_and_fail;
+      CHECK_DEAD_GOTO (pulsesink, unlock_and_fail);
 
       if ((l = pa_stream_writable_size (pulsesink->stream)) == (size_t) - 1) {
         GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
@@ -899,9 +545,6 @@ gst_pulsesink_write (GstAudioSink * asink, gpointer data, guint length)
       if (l > 0)
         break;
 
-      if (pulsesink->did_reset)
-        goto unlock_and_fail;
-
       pa_threaded_mainloop_wait (pulsesink->mainloop);
     }
 
@@ -922,32 +565,27 @@ gst_pulsesink_write (GstAudioSink * asink, gpointer data, guint length)
     sum += l;
   }
 
-  pulsesink->did_reset = FALSE;
-  pulsesink->in_write = FALSE;
-
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
+
   return sum;
 
 unlock_and_fail:
 
-  pulsesink->did_reset = FALSE;
-  pulsesink->in_write = FALSE;
-
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
-  return (guint) - 1;
+  return 0;
 }
 
 static guint
 gst_pulsesink_delay (GstAudioSink * asink)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (asink);
+
   pa_usec_t t;
 
   pa_threaded_mainloop_lock (pulsesink->mainloop);
 
   for (;;) {
-    if (gst_pulsesink_is_dead (pulsesink))
-      goto unlock_and_fail;
+    CHECK_DEAD_GOTO (pulsesink, unlock_and_fail);
 
     if (pa_stream_get_latency (pulsesink->stream, &t, NULL) >= 0)
       break;
@@ -977,7 +615,7 @@ gst_pulsesink_success_cb (pa_stream * s, int success, void *userdata)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (userdata);
 
-  pulsesink->operation_success = !!success;
+  pulsesink->operation_success = success;
   pa_threaded_mainloop_signal (pulsesink->mainloop, 0);
 }
 
@@ -985,12 +623,12 @@ static void
 gst_pulsesink_reset (GstAudioSink * asink)
 {
   GstPulseSink *pulsesink = GST_PULSESINK (asink);
+
   pa_operation *o = NULL;
 
   pa_threaded_mainloop_lock (pulsesink->mainloop);
 
-  if (gst_pulsesink_is_dead (pulsesink))
-    goto unlock_and_fail;
+  CHECK_DEAD_GOTO (pulsesink, unlock_and_fail);
 
   if (!(o =
           pa_stream_flush (pulsesink->stream, gst_pulsesink_success_cb,
@@ -1001,17 +639,9 @@ gst_pulsesink_reset (GstAudioSink * asink)
     goto unlock_and_fail;
   }
 
-  /* Inform anyone waiting in _write() call that it shall wakeup */
-  if (pulsesink->in_write) {
-    pulsesink->did_reset = TRUE;
-    pa_threaded_mainloop_signal (pulsesink->mainloop, 0);
-  }
-
-  pulsesink->operation_success = FALSE;
-  while (pa_operation_get_state (o) == PA_OPERATION_RUNNING) {
-
-    if (gst_pulsesink_is_dead (pulsesink))
-      goto unlock_and_fail;
+  pulsesink->operation_success = 0;
+  while (pa_operation_get_state (o) != PA_OPERATION_DONE) {
+    CHECK_DEAD_GOTO (pulsesink, unlock_and_fail);
 
     pa_threaded_mainloop_wait (pulsesink->mainloop);
   }
@@ -1042,21 +672,25 @@ gst_pulsesink_change_title (GstPulseSink * pulsesink, const gchar * t)
   g_free (pulsesink->stream_name);
   pulsesink->stream_name = g_strdup (t);
 
-  if (gst_pulsesink_is_dead (pulsesink))
-    goto unlock;
+  if (!(pulsesink)->context
+      || pa_context_get_state ((pulsesink)->context) != PA_CONTEXT_READY
+      || !(pulsesink)->stream
+      || pa_stream_get_state ((pulsesink)->stream) != PA_STREAM_READY) {
+    goto unlock_and_fail;
+  }
 
   if (!(o =
           pa_stream_set_name (pulsesink->stream, pulsesink->stream_name, NULL,
-              NULL))) {
+              pulsesink))) {
     GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
         ("pa_stream_set_name() failed: %s",
             pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-    goto unlock;
+    goto unlock_and_fail;
   }
 
   /* We're not interested if this operation failed or not */
 
-unlock:
+unlock_and_fail:
 
   if (o)
     pa_operation_unref (o);
@@ -1064,74 +698,6 @@ unlock:
   pa_threaded_mainloop_unlock (pulsesink->mainloop);
 }
 
-#if HAVE_PULSE_0_9_11
-static void
-gst_pulsesink_change_props (GstPulseSink * pulsesink, GstTagList * l)
-{
-
-  static const gchar *const map[] = {
-    GST_TAG_TITLE, PA_PROP_MEDIA_TITLE,
-    GST_TAG_ARTIST, PA_PROP_MEDIA_ARTIST,
-    GST_TAG_LANGUAGE_CODE, PA_PROP_MEDIA_LANGUAGE,
-    GST_TAG_LOCATION, PA_PROP_MEDIA_FILENAME,
-    /* We might add more here later on ... */
-    NULL
-  };
-
-  pa_proplist *pl = NULL;
-  const gchar *const *t;
-  gboolean empty = TRUE;
-  pa_operation *o = NULL;
-
-  pl = pa_proplist_new ();
-
-  for (t = map; *t; t += 2) {
-    gchar *n = NULL;
-
-    if (gst_tag_list_get_string (l, *t, &n)) {
-
-      if (n && *n) {
-        pa_proplist_sets (pl, *(t + 1), n);
-        empty = FALSE;
-      }
-
-      g_free (n);
-    }
-  }
-
-  if (empty)
-    goto finish;
-
-  pa_threaded_mainloop_lock (pulsesink->mainloop);
-
-  if (gst_pulsesink_is_dead (pulsesink))
-    goto unlock;
-
-  if (!(o =
-          pa_stream_proplist_update (pulsesink->stream, PA_UPDATE_REPLACE, pl,
-              NULL, NULL))) {
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("pa_stream_proplist_update() failed: %s",
-            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-    goto unlock;
-  }
-
-  /* We're not interested if this operation failed or not */
-
-unlock:
-
-  if (o)
-    pa_operation_unref (o);
-
-  pa_threaded_mainloop_unlock (pulsesink->mainloop);
-
-finish:
-
-  if (pl)
-    pa_proplist_free (pl);
-}
-#endif
-
 static gboolean
 gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
 {
@@ -1170,10 +736,6 @@ gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
       g_free (description);
       g_free (buf);
 
-#if HAVE_PULSE_0_9_11
-      gst_pulsesink_change_props (pulsesink, l);
-#endif
-
       break;
     }
     default:
@@ -1183,61 +745,28 @@ gst_pulsesink_event (GstBaseSink * sink, GstEvent * event)
   return GST_BASE_SINK_CLASS (parent_class)->event (sink, event);
 }
 
-static void
-gst_pulsesink_pause (GstPulseSink * pulsesink, gboolean b)
+GType
+gst_pulsesink_get_type (void)
 {
-  pa_operation *o = NULL;
+  static GType pulsesink_type = 0;
 
-  pa_threaded_mainloop_lock (pulsesink->mainloop);
+  if (!pulsesink_type) {
 
-  if (gst_pulsesink_is_dead (pulsesink))
-    goto unlock;
+    static const GTypeInfo pulsesink_info = {
+      sizeof (GstPulseSinkClass),
+      gst_pulsesink_base_init,
+      NULL,
+      gst_pulsesink_class_init,
+      NULL,
+      NULL,
+      sizeof (GstPulseSink),
+      0,
+      gst_pulsesink_init,
+    };
 
-  if (!(o = pa_stream_cork (pulsesink->stream, b, NULL, NULL))) {
-
-    GST_ELEMENT_ERROR (pulsesink, RESOURCE, FAILED,
-        ("pa_stream_cork() failed: %s",
-            pa_strerror (pa_context_errno (pulsesink->context))), (NULL));
-    goto unlock;
+    pulsesink_type = g_type_register_static (GST_TYPE_AUDIO_SINK,
+        "GstPulseSink", &pulsesink_info, 0);
   }
 
-  while (pa_operation_get_state (o) == PA_OPERATION_RUNNING) {
-
-    if (gst_pulsesink_is_dead (pulsesink))
-      goto unlock;
-
-    pa_threaded_mainloop_wait (pulsesink->mainloop);
-  }
-
-unlock:
-
-  if (o)
-    pa_operation_unref (o);
-
-  pa_threaded_mainloop_unlock (pulsesink->mainloop);
-}
-
-
-static GstStateChangeReturn
-gst_pulsesink_change_state (GstElement * element, GstStateChange transition)
-{
-  GstPulseSink *this = GST_PULSESINK (element);
-
-  switch (transition) {
-
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-
-      gst_pulsesink_pause (this,
-          GST_STATE_TRANSITION_NEXT (transition) == GST_STATE_PAUSED);
-      break;
-
-    default:
-      ;
-  }
-
-  if (GST_ELEMENT_CLASS (parent_class)->change_state)
-    return GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-
-  return GST_STATE_CHANGE_SUCCESS;
+  return pulsesink_type;
 }
