@@ -22,14 +22,18 @@
 /**
  * SECTION:element-dv1394src
  *
- * Read DV (digital video) data from firewire port.
- *
  * <refsect2>
+ * <para>
+ * Read DV (digital video) data from firewire port.
+ * </para>
  * <title>Example launch line</title>
- * |[
- * gst-launch dv1394src ! queue ! dvdemux name=d ! queue ! dvdec ! xvimagesink d. ! queue ! alsasink
- * ]| This pipeline captures from the firewire port and displays it (might need
+ * <para>
+ * <programlisting>
+ * gst-launch dv1394src ! dvdemux name=d ! queue ! dvdec ! xvimagesink d. ! queue ! alsasink
+ * </programlisting>
+ * This pipeline captures from the firewire port and displays it (might need
  * format converters for audio/video).
+ * </para>
  * </refsect2>
  */
 
@@ -42,7 +46,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <string.h>
-#include <stdlib.h>
 
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
@@ -56,7 +59,6 @@
 
 #include "gstdv1394src.h"
 #include "gst1394probe.h"
-#include "gst1394clock.h"
 
 
 #define CONTROL_STOP            'S'     /* stop the select call */
@@ -66,8 +68,8 @@
 
 #define SEND_COMMAND(src, command)          \
 G_STMT_START {                              \
-  int G_GNUC_UNUSED _res; unsigned char c; c = command;   \
-  _res = write (WRITE_SOCKET(src), &c, 1);  \
+  unsigned char c; c = command;             \
+  write (WRITE_SOCKET(src), &c, 1);         \
 } G_STMT_END
 
 #define READ_COMMAND(src, command, res)        \
@@ -113,6 +115,14 @@ enum
   PROP_DEVICE_NAME
 };
 
+static const GstElementDetails gst_dv1394src_details =
+GST_ELEMENT_DETAILS ("Firewire (1394) DV video source",
+    "Source/Video",
+    "Source for DV video data from firewire port",
+    "Erik Walthinsen <omega@temple-baptist.com>\n"
+    "Daniel Fischer <dan@f3c.com>\n" "Wim Taymans <wim@fluendo.com>\n"
+    "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
+
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
@@ -129,17 +139,18 @@ static void gst_dv1394src_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_dv1394src_dispose (GObject * object);
 
-static GstClock *gst_dv1394src_provide_clock (GstElement * element);
-static GstStateChangeReturn gst_dv1394_src_change_state (GstElement * element,
-    GstStateChange transition);
-
 static gboolean gst_dv1394src_start (GstBaseSrc * bsrc);
 static gboolean gst_dv1394src_stop (GstBaseSrc * bsrc);
 static gboolean gst_dv1394src_unlock (GstBaseSrc * bsrc);
 
 static GstFlowReturn gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf);
 
-static gboolean gst_dv1394src_query (GstBaseSrc * src, GstQuery * query);
+static gboolean gst_dv1394src_convert (GstPad * pad,
+    GstFormat src_format, gint64 src_value,
+    GstFormat * dest_format, gint64 * dest_value);
+
+static const GstQueryType *gst_dv1394src_get_query_types (GstPad * pad);
+static gboolean gst_dv1394src_query (GstPad * pad, GstQuery * query);
 static void gst_dv1394src_update_device_name (GstDV1394Src * src);
 
 static void
@@ -170,26 +181,20 @@ gst_dv1394src_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_static_pad_template (element_class, &src_factory);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&src_factory));
 
-  gst_element_class_set_details_simple (element_class,
-      "Firewire (1394) DV video source", "Source/Video",
-      "Source for DV video data from firewire port",
-      "Erik Walthinsen <omega@temple-baptist.com>, "
-      "Daniel Fischer <dan@f3c.com>, " "Wim Taymans <wim@fluendo.com>, "
-      "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
+  gst_element_class_set_details (element_class, &gst_dv1394src_details);
 }
 
 static void
 gst_dv1394src_class_init (GstDV1394SrcClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
   GstPushSrcClass *gstpushsrc_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
   gstpushsrc_class = (GstPushSrcClass *) klass;
 
@@ -197,9 +202,6 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
   gobject_class->get_property = gst_dv1394src_get_property;
   gobject_class->dispose = gst_dv1394src_dispose;
 
-  gstelement_class->provide_clock = gst_dv1394src_provide_clock;
-  gstelement_class->change_state = gst_dv1394_src_change_state;
-
   gst_dv1394src_signals[SIGNAL_FRAME_DROPPED] =
       g_signal_new ("frame-dropped", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_LAST, G_STRUCT_OFFSET (GstDV1394SrcClass, frame_dropped),
@@ -207,30 +209,29 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port", "Port number (-1 automatic)",
-          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          -1, 16, DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CHANNEL,
       g_param_spec_int ("channel", "Channel", "Channel number for listening",
-          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, 64, DEFAULT_CHANNEL, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_CONSECUTIVE,
       g_param_spec_int ("consecutive", "consecutive frames",
           "send n consecutive frames after skipping", 1, G_MAXINT,
-          DEFAULT_CONSECUTIVE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_CONSECUTIVE, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_SKIP,
       g_param_spec_int ("skip", "skip frames", "skip n frames",
-          0, G_MAXINT, DEFAULT_SKIP,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          0, G_MAXINT, DEFAULT_SKIP, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DROP_INCOMPLETE,
-      g_param_spec_boolean ("drop-incomplete", "drop incomplete",
+      g_param_spec_boolean ("drop_incomplete", "drop_incomplete",
           "drop incomplete frames", DEFAULT_DROP_INCOMPLETE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_USE_AVC,
       g_param_spec_boolean ("use-avc", "Use AV/C", "Use AV/C VTR control",
-          DEFAULT_USE_AVC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_USE_AVC, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_GUID,
       g_param_spec_uint64 ("guid", "GUID",
           "select one of multiple DV devices by its GUID. use a hexadecimal "
           "like 0xhhhhhhhhhhhhhhhh. (0 = no guid)", 0, G_MAXUINT64,
-          DEFAULT_GUID, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_GUID, G_PARAM_READWRITE));
   /**
    * GstDV1394Src:device-name
    *
@@ -240,14 +241,12 @@ gst_dv1394src_class_init (GstDV1394SrcClass * klass)
    **/
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "device name",
-          "user-friendly name of the device", "Default",
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          "user-friendly name of the device", "Default", G_PARAM_READABLE));
 
   gstbasesrc_class->negotiate = NULL;
   gstbasesrc_class->start = gst_dv1394src_start;
   gstbasesrc_class->stop = gst_dv1394src_stop;
   gstbasesrc_class->unlock = gst_dv1394src_unlock;
-  gstbasesrc_class->query = gst_dv1394src_query;
 
   gstpushsrc_class->create = gst_dv1394src_create;
 }
@@ -258,10 +257,11 @@ gst_dv1394src_init (GstDV1394Src * dv1394src, GstDV1394SrcClass * klass)
   GstPad *srcpad = GST_BASE_SRC_PAD (dv1394src);
 
   gst_base_src_set_live (GST_BASE_SRC (dv1394src), TRUE);
-  gst_base_src_set_format (GST_BASE_SRC (dv1394src), GST_FORMAT_TIME);
-  gst_base_src_set_do_timestamp (GST_BASE_SRC (dv1394src), TRUE);
   gst_pad_use_fixed_caps (srcpad);
 
+  gst_pad_set_query_function (srcpad, gst_dv1394src_query);
+  gst_pad_set_query_type_function (srcpad, gst_dv1394src_get_query_types);
+
   dv1394src->port = DEFAULT_PORT;
   dv1394src->channel = DEFAULT_CHANNEL;
 
@@ -282,8 +282,6 @@ gst_dv1394src_init (GstDV1394Src * dv1394src, GstDV1394SrcClass * klass)
   dv1394src->buf = NULL;
   dv1394src->frame = NULL;
   dv1394src->frame_sequence = 0;
-
-  dv1394src->provided_clock = gst_1394_clock_new ("dv1394clock");
 }
 
 static void
@@ -291,10 +289,6 @@ gst_dv1394src_dispose (GObject * object)
 {
   GstDV1394Src *src = GST_DV1394SRC (object);
 
-  if (src->provided_clock) {
-    g_object_unref (src->provided_clock);
-  }
-
   g_free (src->uri);
   src->uri = NULL;
 
@@ -377,47 +371,6 @@ gst_dv1394src_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
-static GstClock *
-gst_dv1394src_provide_clock (GstElement * element)
-{
-  GstDV1394Src *dv1394src = GST_DV1394SRC (element);
-
-  return GST_CLOCK_CAST (gst_object_ref (dv1394src->provided_clock));
-}
-
-static GstStateChangeReturn
-gst_dv1394_src_change_state (GstElement * element, GstStateChange transition)
-{
-  GstStateChangeReturn ret = GST_STATE_CHANGE_SUCCESS;
-  GstDV1394Src *src = GST_DV1394SRC (element);
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PLAYING_TO_PAUSED:
-      gst_element_post_message (element,
-          gst_message_new_clock_lost (GST_OBJECT_CAST (element),
-              GST_CLOCK_CAST (src->provided_clock)));
-      break;
-    default:
-      break;
-  }
-
-  ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
-  if (ret == GST_STATE_CHANGE_FAILURE)
-    return ret;
-
-  switch (transition) {
-    case GST_STATE_CHANGE_PAUSED_TO_PLAYING:
-      gst_element_post_message (element,
-          gst_message_new_clock_provide (GST_OBJECT_CAST (element),
-              GST_CLOCK_CAST (src->provided_clock), TRUE));
-      break;
-    default:
-      break;
-  }
-
-  return ret;
-}
-
 #ifdef HAVE_LIBIEC61883
 static GstDV1394Src *
 gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
@@ -442,7 +395,7 @@ gst_dv1394src_iec61883_receive (unsigned char *data, int len,
 {
   GstDV1394Src *dv1394src = GST_DV1394SRC (cbdata);
 
-  if (G_UNLIKELY (!GST_PAD_CAPS (GST_BASE_SRC_PAD (dv1394src)))) {
+  if (!GST_PAD_CAPS (GST_BASE_SRC_PAD (dv1394src))) {
     GstCaps *caps;
     unsigned char *p = data;
 
@@ -468,17 +421,26 @@ gst_dv1394src_iec61883_receive (unsigned char *data, int len,
     gst_pad_set_caps (GST_BASE_SRC_PAD (dv1394src), caps);
     gst_caps_unref (caps);
   }
-
   dv1394src->frame = NULL;
-  if (G_LIKELY ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
-              dv1394src->consecutive) < dv1394src->consecutive)) {
+  if ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
+          dv1394src->consecutive) < dv1394src->consecutive) {
     if (complete && len == dv1394src->frame_size) {
+      gint64 i64;
       guint8 *bufdata;
       GstBuffer *buf;
+      GstFormat format;
 
       buf = gst_buffer_new_and_alloc (dv1394src->frame_size);
 
+      /* fill in offset, duration, timestamp */
       GST_BUFFER_OFFSET (buf) = dv1394src->frame_sequence;
+      format = GST_FORMAT_TIME;
+      gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
+          GST_BUFFER_OFFSET (buf), &format, &i64);
+      GST_BUFFER_TIMESTAMP (buf) = i64;
+      gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
+          1, &format, &i64);
+      GST_BUFFER_DURATION (buf) = i64;
       bufdata = GST_BUFFER_DATA (buf);
       memcpy (bufdata, data, len);
       dv1394src->buf = buf;
@@ -550,6 +512,7 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
       }
       if ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
               dv1394src->consecutive) < dv1394src->consecutive) {
+        GstFormat format;
         GstBuffer *buf;
         gint64 i64;
 
@@ -557,6 +520,14 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
 
         /* fill in offset, duration, timestamp */
         GST_BUFFER_OFFSET (buf) = dv1394src->frame_sequence;
+        format = GST_FORMAT_TIME;
+        gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
+            GST_BUFFER_OFFSET (buf), &format, &i64);
+        GST_BUFFER_TIMESTAMP (buf) = i64;
+        gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
+            1, &format, &i64);
+        GST_BUFFER_DURATION (buf) = i64;
+
         dv1394src->frame = buf;
       }
       dv1394src->frame_sequence++;
@@ -674,7 +645,7 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
   pollfds[1].fd = READ_SOCKET (dv1394src);
   pollfds[1].events = POLLIN | POLLERR | POLLHUP | POLLPRI;
 
-  if (G_UNLIKELY (dv1394src->buf)) {
+  if (dv1394src->buf) {
     /* maybe we had an error before, and there's a stale buffer? */
     gst_buffer_unref (dv1394src->buf);
     dv1394src->buf = NULL;
@@ -683,13 +654,12 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
   while (TRUE) {
     int res = poll (pollfds, 2, -1);
 
-    if (G_UNLIKELY (res < 0)) {
+    if (res < 0) {
       if (errno == EAGAIN || errno == EINTR)
         continue;
       else
         goto error_while_polling;
     }
-
     if (G_UNLIKELY (pollfds[1].revents)) {
       char command;
 
@@ -744,6 +714,7 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
   /* loop over all our ports */
   for (; j < m && node == -1; j++) {
     raw1394handle_t handle;
+    gint n_ports;
     struct raw1394_portinfo pinf[16];
 
     /* open the port */
@@ -752,7 +723,7 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
       GST_WARNING ("raw1394 - failed to get handle: %s.\n", strerror (errno));
       continue;
     }
-    if (raw1394_get_port_info (handle, pinf, 16) < 0) {
+    if ((n_ports = raw1394_get_port_info (handle, pinf, 16)) < 0) {
       GST_WARNING ("raw1394 - failed to get port info: %s.\n",
           strerror (errno));
       goto next;
@@ -888,8 +859,6 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
     }
   }
 
-  gst_1394_clock_set_handle (src->provided_clock, src->handle);
-
   return TRUE;
 
 socket_pair:
@@ -984,8 +953,6 @@ gst_dv1394src_stop (GstBaseSrc * bsrc)
     }
   }
 
-  gst_1394_clock_unset_handle (src->provided_clock);
-
   raw1394_destroy_handle (src->handle);
 
   return TRUE;
@@ -1002,22 +969,122 @@ gst_dv1394src_unlock (GstBaseSrc * bsrc)
 }
 
 static gboolean
-gst_dv1394src_query (GstBaseSrc * basesrc, GstQuery * query)
+gst_dv1394src_convert (GstPad * pad,
+    GstFormat src_format, gint64 src_value,
+    GstFormat * dest_format, gint64 * dest_value)
 {
-  switch (GST_QUERY_TYPE (query)) {
-    case GST_QUERY_LATENCY:
-    {
-      gst_query_set_latency (query, TRUE, GST_SECOND / 25, GST_CLOCK_TIME_NONE);
-    }
+  GstDV1394Src *src;
+
+  src = GST_DV1394SRC (gst_pad_get_parent (pad));
+
+  switch (src_format) {
+    case GST_FORMAT_TIME:
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          src_value *= src->frame_size;
+        case GST_FORMAT_DEFAULT:
+          *dest_value =
+              gst_util_uint64_scale_int (src_value, src->frame_rate,
+              GST_SECOND);
+          break;
+        default:
+          goto not_supported;
+      }
+      break;
+    case GST_FORMAT_BYTES:
+      src_value /= src->frame_size;
+    case GST_FORMAT_DEFAULT:
+      switch (*dest_format) {
+        case GST_FORMAT_BYTES:
+          *dest_value = src_value * src->frame_size;
+          break;
+        case GST_FORMAT_TIME:
+          if (src->frame_rate != 0)
+            *dest_value =
+                gst_util_uint64_scale_int (src_value, GST_SECOND,
+                src->frame_rate);
+          else
+            goto not_supported;
+          break;
+        default:
+          goto not_supported;
+      }
       break;
     default:
       goto not_supported;
   }
 
+  gst_object_unref (src);
   return TRUE;
 
 not_supported:
-  return GST_BASE_SRC_CLASS (parent_class)->query (basesrc, query);
+  {
+    GST_DEBUG_OBJECT (src, "unsupported conversion");
+    gst_object_unref (src);
+    return FALSE;
+  }
+}
+
+static const GstQueryType *
+gst_dv1394src_get_query_types (GstPad * pad)
+{
+  static const GstQueryType src_query_types[] = {
+    GST_QUERY_CONVERT,
+    GST_QUERY_POSITION,
+    0
+  };
+
+  return src_query_types;
+}
+
+static gboolean
+gst_dv1394src_query (GstPad * pad, GstQuery * query)
+{
+  gboolean res = TRUE;
+  GstDV1394Src *src;
+
+  src = GST_DV1394SRC (gst_pad_get_parent (pad));
+
+  switch (GST_QUERY_TYPE (query)) {
+    case GST_QUERY_POSITION:
+    {
+      GstFormat format;
+      gint64 current;
+
+      gst_query_parse_position (query, &format, NULL);
+
+      /* bring our current frame to the requested format */
+      res = gst_pad_query_convert (pad,
+          GST_FORMAT_DEFAULT, src->frame_sequence, &format, &current);
+
+      gst_query_set_position (query, format, current);
+      break;
+    }
+    case GST_QUERY_CONVERT:
+    {
+      GstFormat src_fmt, dest_fmt;
+      gint64 src_val, dest_val;
+
+      gst_query_parse_convert (query, &src_fmt, &src_val, &dest_fmt, &dest_val);
+      if (!(res =
+              gst_dv1394src_convert (pad, src_fmt, src_val, &dest_fmt,
+                  &dest_val)))
+        goto not_supported;
+      gst_query_set_convert (query, src_fmt, src_val, dest_fmt, dest_val);
+      break;
+    }
+    default:
+      goto not_supported;
+  }
+
+  gst_object_unref (src);
+  return TRUE;
+
+not_supported:
+  {
+    gst_object_unref (src);
+    return FALSE;
+  }
 }
 
 static void
@@ -1083,7 +1150,7 @@ gst_dv1394src_uri_get_type (void)
 static gchar **
 gst_dv1394src_uri_get_protocols (void)
 {
-  static gchar *protocols[] = { (char *) "dv", NULL };
+  static gchar *protocols[] = { "dv", NULL };
 
   return protocols;
 }
