@@ -52,7 +52,7 @@
 GST_DEBUG_CATEGORY_STATIC (GST_CAT_DEFAULT);
 
 static const GstElementDetails element_details =
-GST_ELEMENT_DETAILS ("AudioPanorama",
+GST_ELEMENT_DETAILS ("Stereo positioning",
     "Filter/Effect/Audio",
     "Positions audio streams in the stereo panorama",
     "Stefan Kost <ensonic@users.sf.net>");
@@ -169,19 +169,19 @@ static GstFlowReturn gst_audio_panorama_transform (GstBaseTransform * base,
 static GstAudioPanoramaProcessFunc panorama_process_functions[2][2][2] = {
   {
         {(GstAudioPanoramaProcessFunc) gst_audio_panorama_transform_m2s_int,
-            (GstAudioPanoramaProcessFunc)
-              gst_audio_panorama_transform_m2s_int_simple},
+              (GstAudioPanoramaProcessFunc)
+            gst_audio_panorama_transform_m2s_int_simple},
         {(GstAudioPanoramaProcessFunc) gst_audio_panorama_transform_m2s_float,
-            (GstAudioPanoramaProcessFunc)
-              gst_audio_panorama_transform_m2s_float_simple}
+              (GstAudioPanoramaProcessFunc)
+            gst_audio_panorama_transform_m2s_float_simple}
       },
   {
         {(GstAudioPanoramaProcessFunc) gst_audio_panorama_transform_s2s_int,
-            (GstAudioPanoramaProcessFunc)
-              gst_audio_panorama_transform_s2s_int_simple},
+              (GstAudioPanoramaProcessFunc)
+            gst_audio_panorama_transform_s2s_int_simple},
         {(GstAudioPanoramaProcessFunc) gst_audio_panorama_transform_s2s_float,
-            (GstAudioPanoramaProcessFunc)
-              gst_audio_panorama_transform_s2s_float_simple}
+              (GstAudioPanoramaProcessFunc)
+            gst_audio_panorama_transform_s2s_float_simple}
       }
 };
 
@@ -242,12 +242,15 @@ static void
 gst_audio_panorama_init (GstAudioPanorama * filter,
     GstAudioPanoramaClass * klass)
 {
+
   filter->panorama = 0;
   filter->method = METHOD_PSYCHOACOUSTIC;
   filter->width = 0;
   filter->channels = 0;
   filter->format_float = FALSE;
   filter->process = NULL;
+
+  gst_base_transform_set_gap_aware (GST_BASE_TRANSFORM (filter), TRUE);
 }
 
 static gboolean
@@ -631,12 +634,15 @@ gst_audio_panorama_transform (GstBaseTransform * base, GstBuffer * inbuf,
   GstAudioPanorama *filter = GST_AUDIO_PANORAMA (base);
   guint num_samples = GST_BUFFER_SIZE (outbuf) / (2 * filter->width);
 
-  if (!gst_buffer_is_writable (outbuf))
-    return GST_FLOW_OK;
-
   if (GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (outbuf)))
     gst_object_sync_values (G_OBJECT (filter), GST_BUFFER_TIMESTAMP (outbuf));
 
+  if (G_UNLIKELY (GST_BUFFER_FLAG_IS_SET (inbuf, GST_BUFFER_FLAG_GAP))) {
+    GST_BUFFER_FLAG_SET (outbuf, GST_BUFFER_FLAG_GAP);
+    memset (GST_BUFFER_DATA (outbuf), 0, GST_BUFFER_SIZE (outbuf));
+    return GST_FLOW_OK;
+  }
+
   filter->process (filter, GST_BUFFER_DATA (inbuf),
       GST_BUFFER_DATA (outbuf), num_samples);
 
