@@ -29,7 +29,7 @@
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_DICETV,GstDiceTVClass))
 #define GST_IS_DICETV(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_DICETV))
-#define GST_IS_DICETV_CLASS(obj) \
+#define GST_IS_DICETV_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_DICETV))
 
 typedef struct _GstDiceTV GstDiceTV;
@@ -167,7 +167,7 @@ gst_dicetv_transform (GstBaseTransform * trans, GstBuffer * in, GstBuffer * out)
   src = (guint32 *) GST_BUFFER_DATA (in);
   dest = (guint32 *) GST_BUFFER_DATA (out);
 
-  gst_buffer_stamp (out, in);
+  gst_buffer_copy_metadata (out, in, GST_BUFFER_COPY_TIMESTAMPS);
 
   video_width = filter->width;
   g_cube_bits = filter->g_cube_bits;
