@@ -7,12 +7,18 @@
  * (c)2000-2003, by iOS-software.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <math.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <strings.h>
+
+#ifdef HAVE_INTTYPES_H
 #include <inttypes.h>
+#endif
 
 #include "goom.h"
 #include "goom_tools.h"
@@ -38,11 +44,11 @@ static void
 init_buffers (PluginInfo * goomInfo, int buffsize)
 {
   goomInfo->pixel = (guint32 *) malloc (buffsize * sizeof (guint32) + 128);
-  bzero (goomInfo->pixel, buffsize * sizeof (guint32) + 128);
+  memset (goomInfo->pixel, 0, buffsize * sizeof (guint32) + 128);
   goomInfo->back = (guint32 *) malloc (buffsize * sizeof (guint32) + 128);
-  bzero (goomInfo->back, buffsize * sizeof (guint32) + 128);
+  memset (goomInfo->back, 0, buffsize * sizeof (guint32) + 128);
   goomInfo->conv = (Pixel *) malloc (buffsize * sizeof (guint32) + 128);
-  bzero (goomInfo->conv, buffsize * sizeof (guint32) + 128);
+  memset (goomInfo->conv, 0, buffsize * sizeof (guint32) + 128);
 
   goomInfo->outputBuf = goomInfo->conv;
 
