@@ -24,8 +24,6 @@
 
 G_BEGIN_DECLS
 
-#define FOURCC_ftyp     GST_MAKE_FOURCC('f','t','y','p')
-#define FOURCC_qt__     GST_MAKE_FOURCC('q','t',' ',' ')
 #define FOURCC_moov     GST_MAKE_FOURCC('m','o','o','v')
 #define FOURCC_mvhd     GST_MAKE_FOURCC('m','v','h','d')
 #define FOURCC_clip     GST_MAKE_FOURCC('c','l','i','p')
@@ -46,7 +44,6 @@ G_BEGIN_DECLS
 #define FOURCC_mdia     GST_MAKE_FOURCC('m','d','i','a')
 #define FOURCC_mdhd     GST_MAKE_FOURCC('m','d','h','d')
 #define FOURCC_hdlr     GST_MAKE_FOURCC('h','d','l','r')
-#define FOURCC_hndl     GST_MAKE_FOURCC('h','n','d','l')
 #define FOURCC_minf     GST_MAKE_FOURCC('m','i','n','f')
 #define FOURCC_vmhd     GST_MAKE_FOURCC('v','m','h','d')
 #define FOURCC_smhd     GST_MAKE_FOURCC('s','m','h','d')
@@ -58,13 +55,11 @@ G_BEGIN_DECLS
 #define FOURCC_stsd     GST_MAKE_FOURCC('s','t','s','d')
 #define FOURCC_stts     GST_MAKE_FOURCC('s','t','t','s')
 #define FOURCC_stss     GST_MAKE_FOURCC('s','t','s','s')
-#define FOURCC_stps     GST_MAKE_FOURCC('s','t','p','s')
 #define FOURCC_stsc     GST_MAKE_FOURCC('s','t','s','c')
 #define FOURCC_stsz     GST_MAKE_FOURCC('s','t','s','z')
 #define FOURCC_stco     GST_MAKE_FOURCC('s','t','c','o')
 #define FOURCC_vide     GST_MAKE_FOURCC('v','i','d','e')
 #define FOURCC_soun     GST_MAKE_FOURCC('s','o','u','n')
-#define FOURCC_subp     GST_MAKE_FOURCC('s','u','b','p')
 #define FOURCC_strm     GST_MAKE_FOURCC('s','t','r','m')
 #define FOURCC_rtsp     GST_MAKE_FOURCC('r','t','s','p')
 #define FOURCC_co64     GST_MAKE_FOURCC('c','o','6','4')
@@ -77,7 +72,6 @@ G_BEGIN_DECLS
 #define FOURCC_wave     GST_MAKE_FOURCC('w','a','v','e')
 #define FOURCC_appl     GST_MAKE_FOURCC('a','p','p','l')
 #define FOURCC_esds     GST_MAKE_FOURCC('e','s','d','s')
-#define FOURCC_pasp     GST_MAKE_FOURCC('p','a','s','p')
 #define FOURCC_hnti     GST_MAKE_FOURCC('h','n','t','i')
 #define FOURCC_rtp_     GST_MAKE_FOURCC('r','t','p',' ')
 #define FOURCC_sdp_     GST_MAKE_FOURCC('s','d','p',' ')
@@ -86,39 +80,19 @@ G_BEGIN_DECLS
 #define FOURCC__nam     GST_MAKE_FOURCC(0xa9,'n','a','m')
 #define FOURCC__cmt     GST_MAKE_FOURCC(0xa9,'c','m','t')
 #define FOURCC__des     GST_MAKE_FOURCC(0xa9,'d','e','s')
-#define FOURCC_desc     GST_MAKE_FOURCC('d','e','s','c')
-#define FOURCC__lyr     GST_MAKE_FOURCC(0xa9,'l','y','r')
 #define FOURCC__cpy     GST_MAKE_FOURCC(0xa9,'c','p','y')
 #define FOURCC__ART     GST_MAKE_FOURCC(0xa9,'A','R','T')
-#define FOURCC_aART     GST_MAKE_FOURCC('a','A','R','T')
 #define FOURCC__wrt     GST_MAKE_FOURCC(0xa9,'w','r','t')
 #define FOURCC__grp     GST_MAKE_FOURCC(0xa9,'g','r','p')
 #define FOURCC__alb     GST_MAKE_FOURCC(0xa9,'a','l','b')
 #define FOURCC__day     GST_MAKE_FOURCC(0xa9,'d','a','y')
-#define FOURCC__req     GST_MAKE_FOURCC(0xa9,'r','e','q')
-#define FOURCC__enc     GST_MAKE_FOURCC(0xa9,'e','n','c')
-#define FOURCC__inf     GST_MAKE_FOURCC(0xa9,'i','n','f')
-#define FOURCC_cprt     GST_MAKE_FOURCC('c','p','r','t')
 #define FOURCC_gnre     GST_MAKE_FOURCC('g','n','r','e')
 #define FOURCC_disc     GST_MAKE_FOURCC('d','i','s','c')
 #define FOURCC_disk     GST_MAKE_FOURCC('d','i','s','k')
 #define FOURCC_trkn     GST_MAKE_FOURCC('t','r','k','n')
 #define FOURCC_cpil     GST_MAKE_FOURCC('c','p','i','l')
-#define FOURCC_pgap     GST_MAKE_FOURCC('p','g','a','p')
-#define FOURCC_pcst     GST_MAKE_FOURCC('p','c','s','t')
 #define FOURCC_tmpo     GST_MAKE_FOURCC('t','m','p','o')
-#define FOURCC_covr     GST_MAKE_FOURCC('c','o','v','r')
 #define FOURCC__too     GST_MAKE_FOURCC(0xa9,'t','o','o')
-#define FOURCC_tvsh     GST_MAKE_FOURCC('t','v','s','h')
-#define FOURCC_tven     GST_MAKE_FOURCC('t','v','e','n')
-#define FOURCC_tvsn     GST_MAKE_FOURCC('t','v','s','n')
-#define FOURCC_tves     GST_MAKE_FOURCC('t','v','e','s')
-#define FOURCC_sonm     GST_MAKE_FOURCC('s','o','n','m')
-#define FOURCC_soal     GST_MAKE_FOURCC('s','o','a','l')
-#define FOURCC_soar     GST_MAKE_FOURCC('s','o','a','r')
-#define FOURCC_soaa     GST_MAKE_FOURCC('s','o','a','a')
-#define FOURCC_soco     GST_MAKE_FOURCC('s','o','c','o')
-#define FOURCC_sosn     GST_MAKE_FOURCC('s','o','s','n')
 #define FOURCC_____     GST_MAKE_FOURCC('-','-','-','-')
 #define FOURCC_free     GST_MAKE_FOURCC('f','r','e','e')
 #define FOURCC_data     GST_MAKE_FOURCC('d','a','t','a')
@@ -131,7 +105,6 @@ G_BEGIN_DECLS
 #define FOURCC_rmvc     GST_MAKE_FOURCC('r','m','v','c')
 #define FOURCC_qtim     GST_MAKE_FOURCC('q','t','i','m')
 #define FOURCC_drms     GST_MAKE_FOURCC('d','r','m','s')
-#define FOURCC_drmi     GST_MAKE_FOURCC('d','r','m','i')
 #define FOURCC_avc1     GST_MAKE_FOURCC('a','v','c','1')
 #define FOURCC_avcC     GST_MAKE_FOURCC('a','v','c','C')
 #define FOURCC_VP31     GST_MAKE_FOURCC('V','P','3','1')
@@ -144,8 +117,6 @@ G_BEGIN_DECLS
 #define FOURCC_twos     GST_MAKE_FOURCC('t','w','o','s')
 #define FOURCC_sowt     GST_MAKE_FOURCC('s','o','w','t')
 #define FOURCC_raw_     GST_MAKE_FOURCC('r','a','w',' ')
-#define FOURCC_in24     GST_MAKE_FOURCC('i','n','2','4')
-#define FOURCC_enda     GST_MAKE_FOURCC('e','n','d','a')
 #define FOURCC_QDM2     GST_MAKE_FOURCC('Q','D','M','2')
 #define FOURCC_alac     GST_MAKE_FOURCC('a','l','a','c')
 #define FOURCC_samr     GST_MAKE_FOURCC('s','a','m','r')
@@ -159,40 +130,6 @@ G_BEGIN_DECLS
 #define FOURCC_url_     GST_MAKE_FOURCC('u','r','l',' ')
 #define FOURCC_frma     GST_MAKE_FOURCC('f','r','m','a')
 #define FOURCC_ctts     GST_MAKE_FOURCC('c','t','t','s')
-#define FOURCC_keyw     GST_MAKE_FOURCC('k','e','y','w')
-#define FOURCC_kywd     GST_MAKE_FOURCC('k','y','w','d')
-#define FOURCC_agsm     GST_MAKE_FOURCC('a','g','s','m')
-#define FOURCC_text     GST_MAKE_FOURCC('t','e','x','t')
-#define FOURCC_tx3g     GST_MAKE_FOURCC('t','x','3','g')
-#define FOURCC_mp4s     GST_MAKE_FOURCC('m','p','4','s')
-#define FOURCC_gama     GST_MAKE_FOURCC('g','a','m','a')
-
-/* SVQ3 fourcc */
-#define FOURCC_SEQH     GST_MAKE_FOURCC('S','E','Q','H')
-#define FOURCC_SMI_     GST_MAKE_FOURCC('S','M','I',' ')
-
-/* 3gpp asset meta data fourcc */
-#define FOURCC_titl     GST_MAKE_FOURCC('t','i','t','l')
-#define FOURCC_dscp     GST_MAKE_FOURCC('d','s','c','p')
-#define FOURCC_perf     GST_MAKE_FOURCC('p','e','r','f')
-#define FOURCC_auth     GST_MAKE_FOURCC('a','u','t','h')
-#define FOURCC_rtng     GST_MAKE_FOURCC('r','t','n','g')
-#define FOURCC_clsf     GST_MAKE_FOURCC('c','l','s','f')
-#define FOURCC_loci     GST_MAKE_FOURCC('l','o','c','i')
-#define FOURCC_albm     GST_MAKE_FOURCC('a','l','b','m')
-#define FOURCC_yrrc     GST_MAKE_FOURCC('y','r','r','c')
-
-/* ISO Motion JPEG 2000 fourcc */
-#define FOURCC_mjp2     GST_MAKE_FOURCC('m','j','p','2')
-#define FOURCC_jp2h     GST_MAKE_FOURCC('j','p','2','h')
-#define FOURCC_ihdr     GST_MAKE_FOURCC('i','h','d','r')
-#define FOURCC_colr     GST_MAKE_FOURCC('c','o','l','r')
-#define FOURCC_fiel     GST_MAKE_FOURCC('f','i','e','l')
-#define FOURCC_jp2x     GST_MAKE_FOURCC('j','p','2','x')
-#define FOURCC_cmap     GST_MAKE_FOURCC('c','m','a','p')
-#define FOURCC_cdef     GST_MAKE_FOURCC('c','d','e','f')
-/* some buggy hardware's notion of mdhd */
-#define FOURCC_mhdr     GST_MAKE_FOURCC('m','h','d','r')
 
 /* Xiph fourcc */
 #define FOURCC_XiTh     GST_MAKE_FOURCC('X','i','T','h')
