@@ -9,7 +9,12 @@
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more 
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -21,26 +26,15 @@
 #include <string.h>
 #include "gstrtpmp4vdepay.h"
 
+GST_DEBUG_CATEGORY_STATIC (rtpmp4vdepay_debug);
+#define GST_CAT_DEFAULT (rtpmp4vdepay_debug)
+
 /* elementfactory information */
-static GstElementDetails gst_rtp_mp4vdepay_details = {
-  "RTP packet parser",
-  "Codec/Depayr/Network",
-  "Extracts MPEG4 video from RTP packets (RFC 3016)",
-  "Wim Taymans <wim@fluendo.com>"
-};
-
-/* RtpMP4VDepay signals and args */
-enum
-{
-  /* FILL ME */
-  LAST_SIGNAL
-};
-
-enum
-{
-  ARG_0,
-  ARG_FREQUENCY
-};
+static const GstElementDetails gst_rtp_mp4vdepay_details =
+GST_ELEMENT_DETAILS ("RTP packet depayloader",
+    "Codec/Depayloader/Network",
+    "Extracts MPEG4 video from RTP packets (RFC 3016)",
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate gst_rtp_mp4v_depay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -56,6 +50,7 @@ GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/x-rtp, "
         "media = (string) \"video\", "
+        "payload = (int) " GST_RTP_PAYLOAD_DYNAMIC_STRING ", "
         "clock-rate = (int) [1, MAX ], " "encoding-name = (string) \"MP4V-ES\""
         /* All optional parameters
          *
@@ -68,16 +63,13 @@ GST_STATIC_PAD_TEMPLATE ("sink",
 GST_BOILERPLATE (GstRtpMP4VDepay, gst_rtp_mp4v_depay, GstBaseRTPDepayload,
     GST_TYPE_BASE_RTP_DEPAYLOAD);
 
+static void gst_rtp_mp4v_depay_finalize (GObject * object);
+
 static gboolean gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload,
     GstCaps * caps);
 static GstBuffer *gst_rtp_mp4v_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static void gst_rtp_mp4v_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_mp4v_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static GstStateChangeReturn gst_rtp_mp4v_depay_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -104,30 +96,42 @@ gst_rtp_mp4v_depay_class_init (GstRtpMP4VDepayClass * klass)
 
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
-
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_BASE_RTP_DEPAYLOAD);
+  gobject_class->finalize = gst_rtp_mp4v_depay_finalize;
+
+  gstelement_class->change_state = gst_rtp_mp4v_depay_change_state;
 
   gstbasertpdepayload_class->process = gst_rtp_mp4v_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_mp4v_depay_setcaps;
 
-  gobject_class->set_property = gst_rtp_mp4v_depay_set_property;
-  gobject_class->get_property = gst_rtp_mp4v_depay_get_property;
-
-  gstelement_class->change_state = gst_rtp_mp4v_depay_change_state;
+  GST_DEBUG_CATEGORY_INIT (rtpmp4vdepay_debug, "rtpmp4vdepay", 0,
+      "MPEG4 video RTP Depayloader");
 }
 
 static void
 gst_rtp_mp4v_depay_init (GstRtpMP4VDepay * rtpmp4vdepay,
     GstRtpMP4VDepayClass * klass)
 {
+  rtpmp4vdepay->adapter = gst_adapter_new ();
+}
+
+static void
+gst_rtp_mp4v_depay_finalize (GObject * object)
+{
+  GstRtpMP4VDepay *rtpmp4vdepay;
+
+  rtpmp4vdepay = GST_RTP_MP4V_DEPAY (object);
+
+  g_object_unref (rtpmp4vdepay->adapter);
+  rtpmp4vdepay->adapter = NULL;
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static gboolean
 gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
-
   GstStructure *structure;
   GstRtpMP4VDepay *rtpmp4vdepay;
   GstCaps *srccaps;
@@ -138,10 +142,8 @@ gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 
   structure = gst_caps_get_structure (caps, 0);
 
-  if (gst_structure_has_field (structure, "clock-rate")) {
+  if (gst_structure_has_field (structure, "clock-rate"))
     gst_structure_get_int (structure, "clock-rate", &clock_rate);
-  }
-
   depayload->clock_rate = clock_rate;
 
   srccaps = gst_caps_new_simple ("video/mpeg",
@@ -159,15 +161,14 @@ gst_rtp_mp4v_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
       gst_buffer_ref (buffer);
       g_value_unset (&v);
 
-      gst_buffer_set_caps (buffer, srccaps);
-
-      gst_caps_unref (srccaps);
-
-      gst_pad_push (depayload->srcpad, buffer);
+      gst_caps_set_simple (srccaps,
+          "codec_data", GST_TYPE_BUFFER, buffer, NULL);
     } else {
       g_warning ("cannot convert config to buffer");
     }
   }
+  gst_pad_set_caps (depayload->srcpad, srccaps);
+  gst_caps_unref (srccaps);
 
   return TRUE;
 }
@@ -183,86 +184,37 @@ gst_rtp_mp4v_depay_process (GstBaseRTPDepayload * depayload, GstBuffer * buf)
   if (!gst_rtp_buffer_validate (buf))
     goto bad_packet;
 
-  {
-    gint payload_len;
-    guint8 *payload;
-    guint32 timestamp;
+  /* flush remaining data on discont */
+  if (GST_BUFFER_IS_DISCONT (buf))
+    gst_adapter_clear (rtpmp4vdepay->adapter);
 
-    payload_len = gst_rtp_buffer_get_payload_len (buf);
-    payload = gst_rtp_buffer_get_payload (buf);
+  outbuf = gst_rtp_buffer_get_payload_buffer (buf);
+  gst_adapter_push (rtpmp4vdepay->adapter, outbuf);
 
-    timestamp = gst_rtp_buffer_get_timestamp (buf);
+  /* if this was the last packet of the VOP, create and push a buffer */
+  if (gst_rtp_buffer_get_marker (buf)) {
+    guint avail;
 
-    outbuf = gst_buffer_new_and_alloc (payload_len);
-    memcpy (GST_BUFFER_DATA (outbuf), payload, payload_len);
+    avail = gst_adapter_available (rtpmp4vdepay->adapter);
 
-    gst_adapter_push (rtpmp4vdepay->adapter, outbuf);
+    outbuf = gst_adapter_take_buffer (rtpmp4vdepay->adapter, avail);
+    gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
 
-    /* if this was the last packet of the VOP, create and push a buffer */
-    if (gst_rtp_buffer_get_marker (buf)) {
-      guint avail;
+    GST_DEBUG ("gst_rtp_mp4v_depay_chain: pushing buffer of size %d",
+        GST_BUFFER_SIZE (outbuf));
 
-      avail = gst_adapter_available (rtpmp4vdepay->adapter);
-
-      outbuf = gst_buffer_new_and_alloc (avail);
-
-      GST_BUFFER_MALLOCDATA (outbuf) =
-          gst_adapter_take (rtpmp4vdepay->adapter, avail);
-      GST_BUFFER_DATA (outbuf) = GST_BUFFER_MALLOCDATA (outbuf);
-      gst_buffer_set_caps (outbuf, GST_PAD_CAPS (depayload->srcpad));
-      GST_BUFFER_TIMESTAMP (outbuf) =
-          timestamp * GST_SECOND / depayload->clock_rate;
-
-      GST_DEBUG ("gst_rtp_mp4v_depay_chain: pushing buffer of size %d",
-          GST_BUFFER_SIZE (outbuf));
-
-      return outbuf;
-    } else {
-      return NULL;
-    }
+    return outbuf;
   }
-
   return NULL;
 
 bad_packet:
   {
     GST_ELEMENT_WARNING (rtpmp4vdepay, STREAM, DECODE,
         ("Packet did not validate"), (NULL));
-
     return NULL;
   }
 }
 
-static void
-gst_rtp_mp4v_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4VDepay *rtpmp4vdepay;
-
-  rtpmp4vdepay = GST_RTP_MP4V_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_mp4v_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec)
-{
-  GstRtpMP4VDepay *rtpmp4vdepay;
-
-  rtpmp4vdepay = GST_RTP_MP4V_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstStateChangeReturn
 gst_rtp_mp4v_depay_change_state (GstElement * element,
     GstStateChange transition)
@@ -273,9 +225,6 @@ gst_rtp_mp4v_depay_change_state (GstElement * element,
   rtpmp4vdepay = GST_RTP_MP4V_DEPAY (element);
 
   switch (transition) {
-    case GST_STATE_CHANGE_NULL_TO_READY:
-      rtpmp4vdepay->adapter = gst_adapter_new ();
-      break;
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       gst_adapter_clear (rtpmp4vdepay->adapter);
       break;
@@ -286,10 +235,6 @@ gst_rtp_mp4v_depay_change_state (GstElement * element,
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
 
   switch (transition) {
-    case GST_STATE_CHANGE_READY_TO_NULL:
-      g_object_unref (rtpmp4vdepay->adapter);
-      rtpmp4vdepay->adapter = NULL;
-      break;
     default:
       break;
   }
@@ -300,5 +245,5 @@ gboolean
 gst_rtp_mp4v_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmp4vdepay",
-      GST_RANK_NONE, GST_TYPE_RTP_MP4V_DEPAY);
+      GST_RANK_MARGINAL, GST_TYPE_RTP_MP4V_DEPAY);
 }
