@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2005> Wim Taymans <wim.taymans@gmail.com>
+ * Copyright (C) <2005> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -52,14 +52,8 @@ struct _GstRtpMP4VPay
   gint          profile;
   GstBuffer    *config;
   gboolean      send_config;
-  gboolean      need_config;
 
-  gboolean      buffer_list;
-
-  /* naming might be confusing with send_config; but naming matches h264
-   * payloader */
-  guint         config_interval;
-  GstClockTime  last_config;
+  GstPadEventFunction old_event_func;
 };
 
 struct _GstRtpMP4VPayClass
@@ -67,8 +61,6 @@ struct _GstRtpMP4VPayClass
   GstBaseRTPPayloadClass parent_class;
 };
 
-GType gst_rtp_mp4v_pay_get_type (void);
-
 gboolean gst_rtp_mp4v_pay_plugin_init (GstPlugin * plugin);
 
 G_END_DECLS
