@@ -1,5 +1,5 @@
 /* GStreamer
- * Copyright (C) <2006> Wim Taymans <wim.taymans@gmail.com>
+ * Copyright (C) <2006> Wim Taymans <wim@fluendo.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -32,15 +32,15 @@ GST_DEBUG_CATEGORY_STATIC (rtpvorbispay_debug);
 #define GST_CAT_DEFAULT (rtpvorbispay_debug)
 
 /* references:
- * http://www.rfc-editor.org/rfc/rfc5215.txt
+ * http://svn.xiph.org/trunk/vorbis/doc/draft-ietf-avt-rtp-vorbis-04.txt
  */
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_vorbispay_details =
-GST_ELEMENT_DETAILS ("RTP Vorbis depayloader",
+GST_ELEMENT_DETAILS ("RTP packet depayloader",
     "Codec/Payloader/Network",
-    "Payload-encode Vorbis audio into RTP packets (RFC 5215)",
-    "Wim Taymans <wimi.taymans@gmail.com>");
+    "Payload-encode Vorbis audio into RTP packets (draft-01 RFC XXXX)",
+    "Wim Taymans <wim@fluendo.com>");
 
 static GstStaticPadTemplate gst_rtp_vorbis_pay_src_template =
 GST_STATIC_PAD_TEMPLATE ("src",
@@ -55,6 +55,11 @@ GST_STATIC_PAD_TEMPLATE ("src",
          * "encoding-params = (string) <num channels>"
          * "configuration = (string) ANY"
          */
+        /* All optional parameters
+         *
+         * "delivery-method = (string) { inline, in_band, out_band/<specific_name> } "
+         * "configuration-uri ="
+         */
     )
     );
 
@@ -91,12 +96,16 @@ gst_rtp_vorbis_pay_base_init (gpointer klass)
 static void
 gst_rtp_vorbis_pay_class_init (GstRtpVorbisPayClass * klass)
 {
+  GObjectClass *gobject_class;
   GstElementClass *gstelement_class;
   GstBaseRTPPayloadClass *gstbasertppayload_class;
 
+  gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
+  parent_class = g_type_class_peek_parent (klass);
+
   gstelement_class->change_state = gst_rtp_vorbis_pay_change_state;
 
   gstbasertppayload_class->set_caps = gst_rtp_vorbis_pay_setcaps;
@@ -218,6 +227,32 @@ gst_rtp_vorbis_pay_flush_packet (GstRtpVorbisPay * rtpvorbispay)
   return ret;
 }
 
+static gchar *
+encode_base64 (const guint8 * in, guint size, guint * len)
+{
+  gchar *ret, *d;
+  static const gchar v[] =
+      "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
+
+  *len = ((size + 2) / 3) * 4;
+  d = ret = (gchar *) g_malloc (*len + 1);
+  for (; size; in += 3) {       /* process tuplets */
+    *d++ = v[in[0] >> 2];       /* byte 1: high 6 bits (1) */
+    /* byte 2: low 2 bits (1), high 4 bits (2) */
+    *d++ = v[((in[0] << 4) + (--size ? (in[1] >> 4) : 0)) & 0x3f];
+    /* byte 3: low 4 bits (2), high 2 bits (3) */
+    *d++ = size ? v[((in[1] << 2) + (--size ? (in[2] >> 6) : 0)) & 0x3f] : '=';
+    /* byte 4: low 6 bits (3) */
+    *d++ = size ? v[in[2] & 0x3f] : '=';
+    if (size)
+      size--;                   /* count third character if processed */
+  }
+  *d = '\0';                    /* tie off string */
+
+  return ret;                   /* return the resulting string */
+}
+
+
 static gboolean
 gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
 {
@@ -227,7 +262,6 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
   gchar *cstr, *configuration;
   guint8 *data, *config;
   guint32 ident;
-  gboolean res;
 
   GST_DEBUG_OBJECT (rtpvorbispay, "finish headers");
 
@@ -334,7 +368,6 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
   for (walk = rtpvorbispay->headers; walk; walk = g_list_next (walk)) {
     GstBuffer *buf = GST_BUFFER_CAST (walk->data);
     guint bsize, size, temp;
-    guint flag;
 
     /* only need to store the length when it's not the last header */
     if (!g_list_next (walk))
@@ -352,12 +385,10 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
 
     bsize = GST_BUFFER_SIZE (buf);
     /* write the size backwards */
-    flag = 0;
     while (size) {
       size--;
-      data[size] = (bsize & 0x7f) | flag;
+      data[size] = bsize & 0x7f;
       bsize >>= 7;
-      flag = 0x80;              /* Flag bit on all bytes of the length except the last */
     }
     data += temp;
   }
@@ -371,20 +402,23 @@ gst_rtp_vorbis_pay_finish_headers (GstBaseRTPPayload * basepayload)
   }
 
   /* serialize to base64 */
-  configuration = g_base64_encode (config, configlen);
+  configuration = encode_base64 (config, configlen, &size);
   g_free (config);
 
   /* configure payloader settings */
   cstr = g_strdup_printf ("%d", rtpvorbispay->channels);
   gst_basertppayload_set_options (basepayload, "audio", TRUE, "VORBIS",
       rtpvorbispay->rate);
-  res =
-      gst_basertppayload_set_outcaps (basepayload, "encoding-params",
-      G_TYPE_STRING, cstr, "configuration", G_TYPE_STRING, configuration, NULL);
+  gst_basertppayload_set_outcaps (basepayload, "encoding-params", G_TYPE_STRING,
+      cstr, "configuration", G_TYPE_STRING, configuration,
+      "delivery-method", G_TYPE_STRING, "inline",
+      /* don't set the other defaults 
+       */
+      NULL);
   g_free (cstr);
   g_free (configuration);
 
-  return res;
+  return TRUE;
 
   /* ERRORS */
 no_headers:
@@ -539,7 +573,7 @@ gst_rtp_vorbis_pay_handle_buffer (GstBaseRTPPayload * basepayload,
   if (rtpvorbispay->packet)
     flush |= (rtpvorbispay->payload_VDT != VDT);
   if (flush)
-    gst_rtp_vorbis_pay_flush_packet (rtpvorbispay);
+    ret = gst_rtp_vorbis_pay_flush_packet (rtpvorbispay);
 
   /* create new packet if we must */
   if (!rtpvorbispay->packet) {
