@@ -23,7 +23,8 @@
 #include <unistd.h>
 
 #include <gst/check/gstcheck.h>
-#include <gst/base/gstadapter.h>
+
+GList *buffers = NULL;
 
 /* For ease of programming we use globals to keep refs for our floating
  * src and sink pads we create; otherwise we always have to do get_pad,
@@ -53,7 +54,7 @@ static GstStaticPadTemplate srcac3template = GST_STATIC_PAD_TEMPLATE ("src",
     GST_STATIC_CAPS (AC3_CAPS_STRING));
 
 
-static GstPad *
+GstPad *
 setup_src_pad (GstElement * element,
     GstStaticPadTemplate * template, GstCaps * caps)
 {
@@ -84,7 +85,7 @@ setup_src_pad (GstElement * element,
   return srcpad;
 }
 
-static void
+void
 teardown_src_pad (GstElement * element)
 {
   GstPad *srcpad, *sinkpad;
@@ -92,14 +93,13 @@ teardown_src_pad (GstElement * element)
   /* clean up floating src pad */
   if (!(sinkpad = gst_element_get_static_pad (element, "audio_0")))
     sinkpad = gst_element_get_request_pad (element, "audio_0");
-  /* references are owned by: 1) us, 2) matroskamux, 3) collect pads */
-  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 3);
+  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
   srcpad = gst_pad_get_peer (sinkpad);
 
   gst_pad_unlink (srcpad, sinkpad);
 
-  /* references are owned by: 1) us, 2) matroskamux, 3) collect pads */
-  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 3);
+  /* pad refs held by both creator and this function (through _get) */
+  ASSERT_OBJECT_REFCOUNT (sinkpad, "sinkpad", 2);
   gst_object_unref (sinkpad);
   /* one more ref is held by element itself */
 
@@ -109,7 +109,7 @@ teardown_src_pad (GstElement * element)
   gst_object_unref (srcpad);
 }
 
-static GstPad *
+GstPad *
 setup_sink_pad (GstElement * element, GstStaticPadTemplate * template,
     GstCaps * caps)
 {
@@ -137,7 +137,7 @@ setup_sink_pad (GstElement * element, GstStaticPadTemplate * template,
   return sinkpad;
 }
 
-static void
+void
 teardown_sink_pad (GstElement * element)
 {
   GstPad *srcpad, *sinkpad;
@@ -159,21 +159,20 @@ teardown_sink_pad (GstElement * element)
 }
 
 
-static GstElement *
+GstElement *
 setup_matroskamux (GstStaticPadTemplate * srctemplate)
 {
   GstElement *matroskamux;
 
   GST_DEBUG ("setup_matroskamux");
   matroskamux = gst_check_setup_element ("matroskamux");
-  g_object_set (matroskamux, "version", 1, NULL);
   mysrcpad = setup_src_pad (matroskamux, srctemplate, NULL);
   mysinkpad = setup_sink_pad (matroskamux, &sinktemplate, NULL);
 
   return matroskamux;
 }
 
-static void
+void
 cleanup_matroskamux (GstElement * matroskamux)
 {
   GST_DEBUG ("cleanup_matroskamux");
@@ -184,7 +183,7 @@ cleanup_matroskamux (GstElement * matroskamux)
   gst_check_teardown_element (matroskamux);
 }
 
-static void
+void
 check_buffer_data (GstBuffer * buffer, void *data, size_t data_size)
 {
   fail_unless (GST_BUFFER_SIZE (buffer) == data_size);
@@ -195,16 +194,19 @@ GST_START_TEST (test_ebml_header)
 {
   GstElement *matroskamux;
   GstBuffer *inbuffer, *outbuffer;
-  GstAdapter *adapter;
   int num_buffers;
   int i;
-  gint available;
-  guint8 data[] =
-      { 0x1a, 0x45, 0xdf, 0xa3, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x14,
-    0x42, 0x82, 0x89, 0x6d, 0x61, 0x74, 0x72, 0x6f, 0x73, 0x6b, 0x61, 0x00,
-    0x42, 0x87, 0x81, 0x01,
-    0x42, 0x85, 0x81, 0x01
+  guint8 data0[12] =
+      { 0x1a, 0x45, 0xdf, 0xa3, 0x01, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
+    0xff
   };
+  guint8 data1[12] =
+      { 0x42, 0x82, 0x89, 0x6d, 0x61, 0x74, 0x72, 0x6f, 0x73, 0x6b, 0x61,
+    0x00
+  };
+  guint8 data2[4] = { 0x42, 0x87, 0x81, 0x01 };
+  guint8 data3[4] = { 0x42, 0x85, 0x81, 0x01 };
+  guint8 data4[8] = { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x14 };
 
   matroskamux = setup_matroskamux (&srcac3template);
   fail_unless (gst_element_set_state (matroskamux,
@@ -215,28 +217,39 @@ GST_START_TEST (test_ebml_header)
   ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
   fail_unless (gst_pad_push (mysrcpad, inbuffer) == GST_FLOW_OK);
   num_buffers = g_list_length (buffers);
-  fail_unless (num_buffers >= 1,
+  fail_unless (num_buffers >= 5,
       "expected at least 5 buffers, but got only %d", num_buffers);
 
-  adapter = gst_adapter_new ();
   for (i = 0; i < num_buffers; ++i) {
     outbuffer = GST_BUFFER (buffers->data);
     fail_if (outbuffer == NULL);
     buffers = g_list_remove (buffers, outbuffer);
 
+    switch (i) {
+      case 0:
+        check_buffer_data (outbuffer, data0, sizeof (data0));
+        break;
+      case 1:
+        check_buffer_data (outbuffer, data1, sizeof (data1));
+        break;
+      case 2:
+        check_buffer_data (outbuffer, data2, sizeof (data2));
+        break;
+      case 3:
+        check_buffer_data (outbuffer, data3, sizeof (data3));
+        break;
+      case 4:
+        check_buffer_data (outbuffer, data4, sizeof (data4));
+        break;
+      default:
+        break;
+    }
+
     ASSERT_BUFFER_REFCOUNT (outbuffer, "outbuffer", 1);
-
-    gst_adapter_push (adapter, outbuffer);
+    gst_buffer_unref (outbuffer);
+    outbuffer = NULL;
   }
 
-  available = gst_adapter_available (adapter);
-  fail_unless (available >= sizeof (data));
-  outbuffer = gst_adapter_take_buffer (adapter, sizeof (data));
-  g_object_unref (adapter);
-
-  check_buffer_data (outbuffer, data, sizeof (data));
-  gst_buffer_unref (outbuffer);
-
   cleanup_matroskamux (matroskamux);
   g_list_free (buffers);
   buffers = NULL;
@@ -273,19 +286,13 @@ GST_START_TEST (test_vorbis_header)
   num_buffers = g_list_length (buffers);
 
   for (i = 0; i < num_buffers; ++i) {
-    gint j;
-
     outbuffer = GST_BUFFER (buffers->data);
     fail_if (outbuffer == NULL);
     buffers = g_list_remove (buffers, outbuffer);
 
-    if (!vorbis_header_found && GST_BUFFER_SIZE (outbuffer) >= sizeof (data)) {
-      for (j = 0; j <= GST_BUFFER_SIZE (outbuffer) - sizeof (data); j++) {
-        if (memcmp (GST_BUFFER_DATA (outbuffer) + j, data, sizeof (data)) == 0) {
-          vorbis_header_found = TRUE;
-          break;
-        }
-      }
+    if (!vorbis_header_found && GST_BUFFER_SIZE (outbuffer) == sizeof (data)) {
+      vorbis_header_found =
+          memcmp (GST_BUFFER_DATA (outbuffer), data, sizeof (data));
     }
 
     ASSERT_BUFFER_REFCOUNT (outbuffer, "outbuffer", 1);
@@ -310,11 +317,11 @@ GST_START_TEST (test_block_group)
   GstCaps *caps;
   int num_buffers;
   int i;
-  guint8 data0[] = { 0xa0, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07,
-    0xa1, 0x85,
-    0x81, 0x00, 0x01, 0x00
-  };
-  guint8 data1[] = { 0x42 };
+  guint8 data0[9] = { 0xa0, 0x01, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff };
+  guint8 data1[2] = { 0xa1, 0x85 };
+  guint8 data2[4] = { 0x81, 0x00, 0x01, 0x00 };
+  guint8 data3[1] = { 0x42 };
+  guint8 data4[8] = { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x07 };
 
   matroskamux = setup_matroskamux (&srcac3template);
   fail_unless (gst_element_set_state (matroskamux,
@@ -356,7 +363,7 @@ GST_START_TEST (test_block_group)
 
   fail_unless (gst_pad_push (mysrcpad, inbuffer) == GST_FLOW_OK);
   num_buffers = g_list_length (buffers);
-  fail_unless (num_buffers >= 2);
+  fail_unless (num_buffers >= 5);
 
   for (i = 0; i < num_buffers; ++i) {
     outbuffer = GST_BUFFER (buffers->data);
@@ -370,6 +377,15 @@ GST_START_TEST (test_block_group)
       case 1:
         check_buffer_data (outbuffer, data1, sizeof (data1));
         break;
+      case 2:
+        check_buffer_data (outbuffer, data2, sizeof (data2));
+        break;
+      case 3:
+        check_buffer_data (outbuffer, data3, sizeof (data3));
+        break;
+      case 4:
+        check_buffer_data (outbuffer, data4, sizeof (data4));
+        break;
       default:
         break;
     }
@@ -387,57 +403,7 @@ GST_START_TEST (test_block_group)
 
 GST_END_TEST;
 
-GST_START_TEST (test_reset)
-{
-  GstElement *matroskamux;
-  GstBuffer *inbuffer;
-  GstBuffer *outbuffer;
-  int num_buffers;
-  int i;
-
-  matroskamux = setup_matroskamux (&srcac3template);
-  fail_unless (gst_element_set_state (matroskamux,
-          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
-      "could not set to playing");
-
-  inbuffer = gst_buffer_new_and_alloc (1);
-  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
-  fail_unless (gst_pad_push (mysrcpad, inbuffer) == GST_FLOW_OK);
-  num_buffers = g_list_length (buffers);
-  fail_unless (num_buffers >= 1,
-      "expected at least 1 buffer, but got only %d", num_buffers);
-
-  fail_unless (gst_element_set_state (matroskamux,
-          GST_STATE_NULL) == GST_STATE_CHANGE_SUCCESS, "could not set to null");
-
-  fail_unless (gst_element_set_state (matroskamux,
-          GST_STATE_PLAYING) == GST_STATE_CHANGE_SUCCESS,
-      "could not set to playing");
-
-  inbuffer = gst_buffer_new_and_alloc (1);
-  ASSERT_BUFFER_REFCOUNT (inbuffer, "inbuffer", 1);
-  fail_unless (gst_pad_push (mysrcpad, inbuffer) == GST_FLOW_OK);
-  num_buffers = g_list_length (buffers);
-  fail_unless (num_buffers >= 2,
-      "expected at least 2 buffers, but got only %d", num_buffers);
-
-  for (i = 0; i < num_buffers; ++i) {
-    outbuffer = GST_BUFFER (buffers->data);
-    fail_if (outbuffer == NULL);
-    buffers = g_list_remove (buffers, outbuffer);
-
-    ASSERT_BUFFER_REFCOUNT (outbuffer, "outbuffer", 1);
-    gst_buffer_unref (outbuffer);
-  }
-
-  cleanup_matroskamux (matroskamux);
-  g_list_free (buffers);
-  buffers = NULL;
-}
-
-GST_END_TEST;
-
-static Suite *
+Suite *
 matroskamux_suite (void)
 {
   Suite *s = suite_create ("matroskamux");
@@ -447,7 +413,6 @@ matroskamux_suite (void)
   tcase_add_test (tc_chain, test_ebml_header);
   tcase_add_test (tc_chain, test_vorbis_header);
   tcase_add_test (tc_chain, test_block_group);
-  tcase_add_test (tc_chain, test_reset);
 
   return s;
 }
